class Main():
    """
        Constructor with no arguments
    """
    def __init__(self):
        pass


    def predict(self,input_file):
        """
        Scan the document and return text

        Parameters
        ----------
        input_file : Byte Array, required
                file to be translated
        
        return: A json object, where keys are fields and values are fields' value. 
        """
        import pdf2image
        import io
        from pdf2image import convert_from_bytes
        import json
        
        
        import os
        import re
        import google.cloud.vision
        from google.cloud import vision
        from pdf2image import convert_from_path
        from google.cloud.vision_v1 import types
        from google.oauth2 import service_account
        credentials = service_account.Credentials.from_service_account_file( './isb-development-274122-17c28d1eb8bd.json' )
        client = vision.ImageAnnotatorClient( credentials = credentials )

        
        bytesPdf = io.BytesIO(input_file)
        images = convert_from_bytes(bytesPdf.read())

        byteIO = io.BytesIO()
        images[0].save(byteIO, format='PNG')
        byteArr = byteIO.getvalue()       
        
        content_image = types.Image(content=byteArr)
        response = client.document_text_detection(image=content_image)
        document = response.full_text_annotation
        
        self.array = document.text.split('\n')
        
        self.createBlock()
        
        self.data = {}
        
        self.data['REGISTRATION'] = self.extractRegistrationInformation(
            self.array[self.block_info['registraion']:self.block_info['patient demographic']])
        
        self.data['PATIENT DEMOGRAPHICS'] = self.extractDemoGraphicInfo(
            self.array[self.block_info['patient demographic']:self.block_info['emergency contact']])

        self.data['EMERGENCY'] = self.extractEmergencyContact(
            self.array[self.block_info['emergency contact']:self.block_info['diagnosis']])
        
        daig = self.array[self.block_info['diagnosis']:self.block_info['physician information']]
        self.data['DIAGNOSIS'] = {'COMPLAINT': self.createKeyValuePair(daig[1],0)}


        
        return json.dumps(self.data)
    
    def createBlock(self):
        self.block_info = {}
        for i in range(len(self.array)):
            if "registration" in self.array[i].lower():
                self.block_info['registraion'] = i
                
            if "patient demographic" in self.array[i].lower():
                self.block_info['patient demographic'] = i

            if "emergency contact" in self.array[i].lower():
                self.block_info['emergency contact'] = i
           
            if "DIAGNOSIS" in self.array[i]:
                self.block_info['diagnosis'] = i

            if "physician information" in self.array[i].lower():
                self.block_info['physician information'] = i
                
            if "PATIENT EMPLOYER INFORMATION" in self.array[i]:
                self.block_info['patient employer information'] = i
                
            if "GUARANTOR INFORMATION" in self.array[i]:
                self.block_info['guarantor information'] = i
                
            if "FINANCIAL INSURANCE" in self.array[i]:
                self.block_info['financial insurance information'] = i
                
    def createKeyValuePair(self,item,sik,isDate=False):
        #split the item using :
        if isDate:
            splitted = item.split(' ')
            value = ' '.join(splitted[sik+1:])
            return value.strip()

        splitted = item.split(':')
        if len(splitted) > 1:
            return splitted[1].strip()
        #split base on space 
        splitted = item.split(' ')
        return ' '.join(splitted[sik+1:]).strip()
    
    def extractRegistrationInformation(self,rawData):
        ext_info = {}
        for idx,item in enumerate(rawData):
            if "MRN" in item:
                ext_info['MRN'] = self.createKeyValuePair(item,0) #item.split(' ')[-1]
            if "VISIT" in item:
                if 'TYPE' in item:
                    ext_info['VISIT TYPE'] = self.createKeyValuePair(item,1) #item.split(' ')[-1]
                else:
                    ext_info['VISIT'] = self.createKeyValuePair(item,0)
            if 'DISCHARGE DATE' in item:
                ext_info['DISCHARGE DATE']  = self.createKeyValuePair(item,1,True)
            if "ADMIT DATE" in item:
                ext_info['ADMIT DATE'] = self.createKeyValuePair(item,1,True)
            if 'READMIT FLAG' in item:
                ext_info['READMIT FLAG'] = self.createKeyValuePair(item,1) #item.split(' ')[-1]
            if 'LAST IP ADMIT' in item:
                ext_info['LAST IP ADMIT'] = self.createKeyValuePair(item,2) #item.split(' ')[-1]
            if 'LAST IP DISCHARGE'  in item:
                ext_info['LAST IP DISCHARGE'] = self.createKeyValuePair(item,2) #item.split(' ')[-1]
            if 'SERVICE' in item:
                ext_info['SERVICE'] = self.createKeyValuePair(item,0)
            if 'PRIOR' in item:
                ext_info['PRIOR DRG/HOSP'] = self.createKeyValuePair(item,1)
            if 'LOCATION' in item:
                ext_info['LOCATION'] = self.createKeyValuePair(item,0)
            if 'CLINICAL RESEARCH' in item:
                ext_info['CLINICAL RESEARCH'] = self.createKeyValuePair(item,1)
            if 'BLOODLESS' in item:
                ext_info['BLOODLESS'] = self.createKeyValuePair(item,0)
            if 'PATIENT NAME' in item:
                ext_info['PATIENT NAME'] = self.createKeyValuePair(item,1)
            if 'CONFIDENTIAL' in item:
                ext_info['CONFIDENTIAL'] = self.createKeyValuePair(item,0)
            if 'MDRO' in item:
                ext_info['MDRO'] = self.createKeyValuePair(item,0)
        return ext_info
    def extractDemoGraphicInfo(self,rawData):
        ext_info = {}
        for idx,item in enumerate(rawData):
            if "MARITAL" in item:
                ext_info['MARITAL'] = self.createKeyValuePair(item,0) #item.split(' ')[-1]
            if 'GENDER' in item:
                ext_info['GENDER']  = self.createKeyValuePair(item,0)
            if "SSN" in item:
                ext_info['SSN'] = self.createKeyValuePair(item,0)
            if 'RELIGION' in item:
                ext_info['RELIGION'] = self.createKeyValuePair(item,0) #item.split(' ')[-1]
            if 'DOB' in item:
                ext_info['DOB'] = self.createKeyValuePair(item,0,True) #item.split(' ')[-1]
            if 'RACE'  in item:
                ext_info['RACE'] = self.createKeyValuePair(item,0) #item.split(' ')[-1]
            if 'LANGUAGE' in item:
                ext_info['LANGUAGE'] = self.createKeyValuePair(item,0)
            if 'ETHNICITY' in item:
                ext_info['ETHNICITY'] = self.createKeyValuePair(item,0)
            if 'ADDRESS' in item:
                ext_info['ADDRESS'] = self.createKeyValuePair(item,0) + ", " +rawData[-4]
            if 'HOME' in item:
                ext_info['HOME'] = self.createKeyValuePair(item,0)
            if 'CELL' in item:
                ext_info['CELL'] = self.createKeyValuePair(item,0)
            if 'NPP' in item:
                ext_info['NPP'] = self.createKeyValuePair(item,0)
            if 'EMAIL' in item:
                ext_info['EMAIL'] = self.createKeyValuePair(item,0)
        return ext_info
    def extractEmergencyContact(self,rawData):
        ext_info = {}
        for idx,item in enumerate(rawData):
            if "NAME" in item:
                ext_info['NAME' + str(idx%2 + 1)] = self.createKeyValuePair(item,0) #item.split(' ')[-1]
            if 'ADDRESS' in item:
                ext_info['ADDRESS'+ str(idx%2 + 1)]  = self.createKeyValuePair(item,0)
            if "RELATION" in item:
                ext_info['RELATION'+ str(idx%2 + 1)] = self.createKeyValuePair(item,0)
            if 'HOME' in item:
                ext_info['HOME'+ str(idx%2 + 1)] = self.createKeyValuePair(item,0) #item.split(' ')[-1]
        return ext_info 

# if __name__ == '__main__':
#    # Test the ML Package locally
#    with open('/home/suleman/101333_Demo.pdf', 'rb') as input_file:
#       byte_data = input_file.read()
#    m = Main()
#    print(m.predict(byte_data))
